// put your BeMob Secret key here (you can find it here: https://panel.bemob.com/app/profile/security)
const LANDING_SECRET_KEY  = '';

// How long signature should be valid.
const SIGNATURE_TTL       = 60 ; // 60 seconds = 1 minute

// GET parameter with BeMob landing signature
const SIGNATURE_GET_PARAM = 'key'; // if you pass your key like this: yourdomain/lp1/?key={landingSignature}

// redirect users that try to access this LP directly to this URL (ie. you can put smartlink URL here)
const REDIRECT_URL        = '';  // ie. https://www.google.com/

// if the REDIRECT_URL is empty, show this safe page HTML to users that try to access this LP directly
const SAFE_PAGE_HTML      = '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><title>Thank You</title></head><body><h1>Thank You!</h1><p>Your submission has been received.</p></body></html>'; 

// in case you don't put an URL in REDIRECT_URL and you don't put a HTML under SAFE_PAGE_HTML the visitor will see a 'Access denied.' message if error occurs.





/* ------------ DO NOT MODIFY BELOW THIS LINE --------------------- */

export default {
  async fetch(request, env) {
    const url = new URL(request.url);
    const searchParams = url.searchParams;
    const pathname = url.pathname;
    const filename = pathname.split('/').pop();


    // Request for a non HTML file? (images, scripts)
    if (filename && !filename.endsWith('.htm') && !filename.endsWith('.html')) {
        return env.ASSETS.fetch(request);
    }

    // LANDING_SECRET_KEY empty?
    if( !LANDING_SECRET_KEY ){
        return new Response('LANDING_SECRET_KEY not defined.', { status: 403 });
    }

    // SIGNATURE_TTL empty?
    if( !SIGNATURE_TTL ){
        return new Response('SIGNATURE_TTL not defined.', { status: 403 });
    }

    const signature = searchParams.get(SIGNATURE_GET_PARAM);
    if (!signature) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }


    const decodedSignature = base64Decode(signature);
    if (!decodedSignature) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    const signatureObject = jsonDecode(decodedSignature);
    if (!signatureObject || !signatureObject.timestamp || !signatureObject.hash) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }


    const signedHash  = await generateHMAC(LANDING_SECRET_KEY, signatureObject.timestamp);
    const currentTime = Date.now();
    if ( signedHash !== signatureObject.hash ) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }

    var timeDiff = (currentTime - signatureObject.timestamp * 1000 + SIGNATURE_TTL) / 1000;
    if ( timeDiff > SIGNATURE_TTL ) {
        if( REDIRECT_URL ){ const redirectResponse = new Response('Redirecting...', { status: 302, headers: { 'Location': REDIRECT_URL } }); return redirectResponse; } else if( SAFE_PAGE_HTML ){ return new Response(SAFE_PAGE_HTML, { headers: { 'Content-Type': 'text/html', }, }); } else { return new Response('Access denied.', { status: 403 }); }
    }


    // Access granted, fetch from env.ASSETS
    return env.ASSETS.fetch(request);

    
  }
};

function base64Decode(encodedString) {
  try {
    return atob(encodedString);
  } catch (error) {
    return null;
  }
}

function jsonDecode(jsonString) {
  try {
    return JSON.parse(jsonString);
  } catch (error) {
    return null;
  }
}


async function generateHMAC(secretKey , timestamp ) {
    const secretKeyArrayBuffer = new TextEncoder().encode(secretKey);
    const timestampArrayBuffer = new TextEncoder().encode(timestamp.toString());

    const importedKey = await crypto.subtle.importKey(
        "raw",
        secretKeyArrayBuffer,
        { name: "HMAC", hash: { name: "SHA-1" } },
        false,
        ["sign"]
    );

    const signatureArrayBuffer = await crypto.subtle.sign(
        "HMAC",
        importedKey,
        timestampArrayBuffer
    );

    const signedHash = Array.prototype.map.call(
        new Uint8Array(signatureArrayBuffer),
        x => ('00' + x.toString(16)).slice(-2)
    ).join('');

    return signedHash;
}

/* ------------ DO NOT MODIFY ABOVE THIS LINE --------------------- */